package org.processmining.causalactivitygraphcreator.creators.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.algorithms.ConvertCausalActivityGraphToActivityClusterArrayAlgorithm;
import org.processmining.activityclusterarraycreator.parameters.ConvertCausalActivityGraphToActivityClusterArrayParameters;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraph.models.impl.CausalActivityGraphFactory;
import org.processmining.causalactivitygraphcreator.creators.GraphCreator;
import org.processmining.causalactivitygraphcreator.creators.GraphCreatorParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.framework.plugin.PluginContext;

public class GraphWeightProbabilityCreator implements GraphCreator {

	public final static String NAME = "Weight Probability";

	private class Parameters {
		int iterations;
		double loValue;
		double hiValue;

		public String toString() {
			return "[iterations: " + iterations + ", low value: " + loValue + ", high value: " + hiValue + "]";
		}
	}

	public CausalActivityGraph apply(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorParameters parameters) {
		CausalActivityGraph bestGraph = null;
		double bestScore = 0.0;
		List<XEventClass> activities = matrix.getActivities();

		Parameters pars = new Parameters();
		pars.iterations = 100000;
		pars.loValue = -0.5;
		pars.hiValue = 0.5;
		parameters.displayMessage("[GraphWeightProbabilityCreator] Parameters: " + pars);

		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(pars.iterations);
		for (int i = 0; i < pars.iterations; i++) {
			CausalActivityGraph graph = CausalActivityGraphFactory.createCausalActivityGraph();
			graph.init(matrix.getLabel(), new HashSet<XEventClass>(activities));
			for (XEventClass source : activities) {
				for (XEventClass target : activities) {
					double value = matrix.getValue(source, target);
					if (isOK(pars, value)) {
						graph.setCausality(source, target, value);
					}
				}
			}
			double score = getScore(context, graph);
			if (score > bestScore) {
				parameters.displayMessage("[GraphWeightProbabilityCreator] Iteration " + i + ", new best score: "
						+ score);
				bestScore = score;
				bestGraph = graph;
			}
			context.getProgress().inc();
		}
		return bestGraph;
	}

	private boolean isOK(Parameters pars, double value) {
		return Math.random() <= ((value - pars.loValue) / (pars.hiValue - pars.loValue));
	}

	private double getScore(PluginContext context, CausalActivityGraph graph) {
		ActivityClusterArray clusters = new ConvertCausalActivityGraphToActivityClusterArrayAlgorithm().apply(context,
				graph, new ConvertCausalActivityGraphToActivityClusterArrayParameters(graph));
		double score = 1.0;
		for (Set<XEventClass> cluster : clusters.getClusters()) {
			double maxSize = Math.max(clusters.getInputs(cluster).size(), clusters.getOutputs(cluster).size());
			score = score * maxSize; // * maxSize;
		}
		List<XEventClass> activities = graph.getActivities();
		for (XEventClass source : activities) {
			for (XEventClass target : activities) {
				score += graph.getCausality(source, target);
			}
		}
		return score;
	}

	public String getName() {
		return NAME;
	}

}
